from pdfminer.pdfparser import PDFParser
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfinterp import resolve1
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams, LTTextBox, LTTextLine, LTFigure
import subprocess, sys, os, pathlib

CONTEXT_COMMAND = ['context', '--purgeresult']
BASEFILENAME = 'xtable-test'
VARIABLESFILE = 'testvariables'
HEADERTEXTS = ['Some Header', 'More Header']
HALTONERROR = True
RETAINPDFS = False

# how many paras on first page
# attn: leave space for the table on first page or test will fail!
FIRSTPAGE_ITER = [ 
    range(1,12), # for using FIRSTPAGE_TEXT[0]
    range(1,3)  # for usingFIRSTPAGE_TEXT[1]
]

# text for para on first page
FIRSTPAGE_TEXT = [
"""Lorem ipsum dolor sit amet, 
consectetur adipiscing elit. Suspendisse et congue nunc. 
Sed congue quam felis, quis consectetur justo scelerisque sed.\\par\\vskip3mm""",

"""Lorem ipsum dolor sit amet, consectetur adipiscing elit. 
Suspendisse et congue nunc. Sed congue quam felis, quis consectetur justo scelerisque sed. 
Donec pellentesque orci id dolor facilisis pulvinar. 
Quisque nisi mi, ornare eu mauris a, rhoncus dictum velit. 
Nulla et venenatis dolor. Cras lacinia faucibus fermentum. 
Quisque iaculis nibh vel lacus elementum ornare. 
Cras maximus magna vel massa lobortis, vitae pretium elit facilisis. 
Aliquam diam mi, pharetra et nunc quis, congue pulvinar lorem. 
Nullam ultricies sodales pellentesque. Nam ac quam et nulla blandit tempor.
Nullam ac tincidunt lacus. Phasellus enim purus, dapibus sit amet ornare et, 
fermentum sed justo. Sed a sagittis sapien.\\par"""
]

# empty space in first page after paras and before table
FIRSTPAGE_SLACK = [ "","\\vskip5mm", "\\vskip5cm" ]

# no of table rows
TABLE_ROW_ITER = [ 
    [5, 8, 10, 20], # for using TABLE_xxx[0]
    [1, 2, 3, 5, 10]  # for using TABLE_xxx[1]
]

# content of table rows
TABLE_LEFT = [
"single line",
"""{\\bf (\\recurselevel) some tale:}
\\startitemize
    \item a quick 
    \item brown fox
    \item jumps over
    \item the lazy dog
\\stopitemize
"""
]
TABLE_RIGHT = [
"two words",
"""{\\bf and a fact:}
\\startitemize
    \item the vodka
    \item is good
    \item but the meat
    \item is rotten
\\stopitemize"""
]

def write_variables(fpiter, fpidx,  slackidx, tbliter, tblidx):
    "creates testvariables.tex file"
    def getAssign(topic, variable, value):
        return ("\n\\doassign["+topic+"]["+variable+"={"+value+"}]").encode('utf-8')

    with open(f"{VARIABLESFILE}.tex", 'wb') as file:
        file.write( getAssign( 'firstpage', 'Iterations', str(fpiter)) )
        file.write( getAssign( 'firstpage', 'Text', FIRSTPAGE_TEXT[fpidx] ) )
        file.write( getAssign( 'firstpage', 'Slack', FIRSTPAGE_SLACK[slackidx]) )

        file.write( getAssign( 'testtable', 'Iterations', str(tbliter)) )
        file.write( getAssign( 'testtable', 'Lefttext', TABLE_LEFT[tblidx]) )
        file.write( getAssign( 'testtable', 'Righttext', TABLE_RIGHT[tblidx]) )


def compile_pdf():
    "calls context"
    cmd = CONTEXT_COMMAND[:]
    cmd.append(f"{BASEFILENAME}.tex")
    print(' '.join(cmd))
    p = subprocess.run(cmd, capture_output=True)
    if p.returncode==0:
        print("...compiled OK")
        return True
    else:
        print("Error compiling PDF")
        print(p.stdout.decode('utf-8'))
        return False


def search_layout(layout, resulz):
    """search this page for the table header's texts"""
    for lt_obj in layout:
        # print(lt_obj.__class__.__name__)
        # print(lt_obj.bbox)
        if isinstance(lt_obj, LTTextBox) or isinstance(lt_obj, LTTextLine):
            text = lt_obj.get_text().strip()
            for idx, hdrtext  in enumerate(HEADERTEXTS):
                # if text == hdrtext:       # doesn't work with slack=="" - PdfMiner mingles LeftHeader with preceding text
                if hdrtext in text:         # so con't compare with but search inside text 
                    resulz[idx] = True
                    if True == all(one for one in resulz):
                        return True 
        elif isinstance(lt_obj, LTFigure):
            return search_layout(lt_obj, resulz)  # Recursive
    return False

def check_document():
    "read PDF and search all pages for the table's header texts"
    file = open(f"{BASEFILENAME}.pdf", 'rb')
    parser = PDFParser(file)
    document = PDFDocument(parser)

    noOfPgs = resolve1(document.catalog['Pages'])['Count']
    print(f"got {noOfPgs} Pages")

    rsrcmgr = PDFResourceManager()
    laparams = LAParams()
    device = PDFPageAggregator(rsrcmgr, laparams=laparams)
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    cnt = 0
    for page in PDFPage.create_pages(document):
        cnt += 1
        print(f"checking pg={cnt}")
        interpreter.process_page(page)
        layout = device.get_result()
        resulz = [False for idx in range(0,len(HEADERTEXTS))]
        if not search_layout(layout, resulz):
            return False
    return True


def create_compile_and_test_one(fpiter, fpidx,  slackidx, tbliter, tblidx):
    write_variables(fpiter, fpidx,  slackidx, tbliter, tblidx)
    if compile_pdf():
        return check_document()
    else:
        return False


def count_no_of_pgs():
    cnt = 0
    for fptextidx in range(0, len(FIRSTPAGE_TEXT)):
        for fpiter in FIRSTPAGE_ITER[fptextidx]:
            for slackidx in range(0, len(FIRSTPAGE_SLACK)):
                for tbltextidx in range(0, len(TABLE_LEFT)):
                    for tbliter in TABLE_ROW_ITER[tbltextidx]:
                        cnt += 1
    return cnt

def run_all_tests():
    for f in pathlib.Path(".").glob(f"{VARIABLESFILE}-err*.tex"):   # remove old error-files
        f.unlink()
    for f in pathlib.Path(".").glob(f"{BASEFILENAME}-err*.pdf"): 
        f.unlink()            
    for f in pathlib.Path(".").glob(f"{BASEFILENAME}-ok*.pdf"):     # remove old success-files
        f.unlink()
    cnt = 0; errs = 0
    allcnt = count_no_of_pgs()
    for fptextidx in range(0, len(FIRSTPAGE_TEXT)):
        for fpiter in FIRSTPAGE_ITER[fptextidx]:
            for slackidx in range(0, len(FIRSTPAGE_SLACK)):
                for tbltextidx in range(0, len(TABLE_LEFT)):
                    for tbliter in TABLE_ROW_ITER[tbltextidx]:
                        cnt += 1
                        print(
                            f"Run {cnt} of {allcnt}: firstpage iter={fpiter}, idx={fptextidx}, "\
                            + f"slackidx={slackidx}, " \
                            + f"table iter={tbliter}, textidx={tbltextidx}"
                        )
                        if not create_compile_and_test_one(fpiter, fptextidx, slackidx, tbliter, tbltextidx):
                            if HALTONERROR:
                                print("ERROR - aborting test-run")
                                sys.exit(-1)
                            else:
                                errs +=1
                                print(f"Error no {errs} - renaming")
                                os.rename(f"{VARIABLESFILE}.tex", f"{VARIABLESFILE}-err{errs}.tex")
                                os.rename(f"{BASEFILENAME}.pdf", f"{BASEFILENAME}-err{errs}.pdf")
                        else:
                            if RETAINPDFS:
                                os.rename(f"{BASEFILENAME}.pdf", f"{BASEFILENAME}-ok{cnt}.pdf")
    if HALTONERROR or errs==0:
        print(f"SUCCESS, all {cnt} tests passed")
    else:
        print(f"ERRORS: {errs} cases of {allcnt} failed, {cnt-errs} succeeded")
        print(f"fail-cases retained, see the {VARIABLESFILE}-err*.tex files")

# ===== MAIN:
# write_variables(5, 1,  0, 5, 1)
run_all_tests()